from dataclasses import dataclass
from pathlib import Path
import jsonref


@dataclass
class GroupDefinition:
    path: str
    type: str
    description: str
    required: bool
    info: dict

    def __str__(self) -> str:
        return f"{self.path}\n- Type: {self.type}\n- Description: {self.description}\n- Required: {self.required}"


def open_schema(
    module: callable,
    path: str,
    image_as_path: bool = False,
):
    json_schema = Path(module.__file__).parent / path
    base_uri = json_schema.parent.absolute().as_uri() + "/"

    with open(str(json_schema)) as schema_file:
        schema = jsonref.load(schema_file, base_uri=base_uri)

    schema_groups = list()
    required_groups = list()
    get_schema_groups(
        schema_groups, schema, required_groups, image_as_path=image_as_path
    )
    return schema_groups


def get_schema_groups(
    schema_groups: list,
    schema_dict: dict,
    required_groups: list,
    base_path: str = "",
    depth: int = 0,
    image_as_path: bool = False,
) -> list[GroupDefinition]:
    for required_path in schema_dict["required"]:
        required_groups.append(f"{base_path}/{required_path}")

    for schema_name in schema_dict["properties"]:
        schema_object = schema_dict["properties"][schema_name]
        schema_type = schema_object["type"]
        path = f"{base_path}/{schema_name}"
        if schema_type == "object":
            for required_path in schema_object["required"]:
                required_groups.append(f"{path}/{required_path}")
            get_schema_groups(
                schema_groups,
                schema_object,
                required_groups,
                path,
                depth + 1,
                image_as_path,
            )
        else:
            if schema_name == "image_path" and not image_as_path:
                schema_type = "image"
                schema_name = "image_array"
                path = f"{base_path}/{schema_name}"
            if schema_name == "timestamp":
                schema_type = "number"
            if path in required_groups:
                required = True
            else:
                required = False
            print(f"{' ' * depth}- {schema_name} {path} {schema_type} {required}")

            group_definition = GroupDefinition(
                path, schema_type, schema_object["description"], required, schema_object
            )
            schema_groups.append(group_definition)
